// Copyright (C) 2001 Jean-Marc Valin

#ifndef UINODE_REPOSITORY_H
#define UINODE_REPOSITORY_H

#include <vector>
#include <string>
#include <map>
#include <set>
#include <libxml/tree.h>

namespace FD {

class UINetwork;

class ValidInfo {
public:
	std::vector<std::string> valid_list;
	bool visible_by_parent;
	ValidInfo() : visible_by_parent(true){}
};

class ItemInfo {
public:
	std::string name;
	std::string type;
	std::string value;
	std::string description;

	// list 配列
	std::vector<std::string> combo_list;
	// stringは valid_list の 対象パラメーター名
	std::map<std::string, ValidInfo*> valid_map;

	~ItemInfo() {
		std::map<std::string, ValidInfo* >::iterator it = valid_map.begin();
		while (it != valid_map.end()) {
			delete it->second;
			it++;
		}
	}
};

class NodeInfo {
public:
	enum NodeKind {builtin, subnet, external};
	std::vector<ItemInfo *> inputs;
	std::vector<ItemInfo *> outputs;
	std::vector<ItemInfo *> params;
	std::string category;
	std::string description;
	std::string sourceFile;
	std::string requireList;
	NodeKind kind;

public:
	NodeInfo() :
		category("Unknown"), description("No description available") {
	}
	~NodeInfo() {
		for (unsigned int i=0; i<inputs.size(); i++)
			delete inputs[i];
		for (unsigned int i=0; i<outputs.size(); i++)
			delete outputs[i];
		for (unsigned int i=0; i<params.size(); i++)
			delete params[i];
	}
private:
	NodeInfo(const NodeInfo&) {
	}

};

class UINodeRepository {
public:
	typedef std::map<std::string, NodeInfo *>::iterator iterator;

private:
	std::map<std::string, NodeInfo *> info;

	static UINodeRepository &GlobalRepository();

	/**List of all files required for each module*/
	static std::map<std::string, std::set<std::string> > &ModuleDepend();

	/**List of all modules required for each file*/
	static std::map<std::string, std::set<std::string> > &FileDepend();

	/**List of all headers required for each file*/
	static std::map<std::string, std::set<std::string> > &HeaderDepend();

	static void LoadAllInfoRecursive(const std::string &path);
	static void LoadNodeDefInfo(const std::string &path,
			const std::string &name);
	static void
			LoadExtDocInfo(const std::string &path, const std::string &name);
	//NodeInfo *&operator[] (const std::string &name) {return info[name];}
public:

	UINodeRepository() {
	}

	UINodeRepository(const UINodeRepository &);

	~UINodeRepository();

	iterator begin() {
		return info.begin();
	}
	iterator end() {
		return info.end();
	}

	NodeInfo *findNode(const std::string &name);

	void loadDocInfo(xmlDocPtr doc, const std::string &basename);

	void loadAllSubnetInfo(xmlNodePtr net);

	void loadNetInfo(xmlNodePtr net);

	void updateNetInfo(UINetwork *net);

	void clean();

	static iterator Begin() {
		return GlobalRepository().info.begin();
	}

	static iterator End() {
		return GlobalRepository().info.end();
	}

	static void Scan();

	static NodeInfo *Find(const std::string &name);

	static std::set<std::string> &FindFileFromModule(const std::string &name);

	static std::set<std::string> &FindModuleFromFile(const std::string &name);

	static std::set<std::string> &FindHeaderFromFile(const std::string &name);

	static std::vector<std::string> Available();

	static void ProcessDependencies(std::set<std::string> &initial_files,
			bool toplevel=true);

	/// Adding for exclusive parameter processing
	/// パラメーターの排他表示用関数
	static void addCombo(std::string node_name, std::string param_name,
			std::string combo_str); /// Add value of list to combo_param_list
	static void printCombo(std::string node_name); /// Debug print
	static void checkCombo(std::string node_name); /// ErrorCheck
	static void addValid(std::string node_name, std::string param_name,
			std::string combo_str); /// Add value of valid to valid_param_name_list
	static void printValid(std::string node_name); /// Debug print
	static void checkValid(std::string node_name); /// ErrorCheck

};
}//namespace FD

#endif
